#!/usr/bin/python3
"""
Скрипт запускается systemd службой при загрузке системы вместо zfs-import-cache.service. Проверяется состояние всех vdev
всех zpool'ов, прописанных в zpool.cache. Если каких-то дисков в составе vdev не хватает и избыточности vdev
недостаточно, чтобы компенсировать недостающие устройства, пул не импортируется. Это делается, чтобы избежать
стандартного поведения zfs при импорте неисправного пула -- kernel oops с последующим отказом работы утилит zfs,
невозможности логина и т.д.
"""
import time
import subprocess
import os
import re
import sys

ZPOOL_CACHE = '/etc/zfs/zpool.cache'
ZDB_INDENT = 4
NEEDED_KEYS = ['type', 'guid', 'pool_guid', 'nparity', 'path', 'create_txg']
DEV_WAIT_TIMEOUT = 10


def fprint(*args, **kwargs):
    print(*args, **kwargs, flush=True)


def zdb_parse(level: int, lines: iter):
    """
    Recursively parses ZDB text output and returns dict with pools, vdevs and disks
    """
    cur_entity = {}
    next_str = ''  # facing entity end in prev.iteration (indent change) we return 1st string of new entity
    while True:
        try:
            line = next_str if next_str else next(lines)
        except StopIteration:
            return cur_entity, ''
        if ':' not in line:
            continue
        key, value = line.split(':', 1)
        _, indent = re.search(r'^[\ ]*', key).span()
        cur_level = indent // ZDB_INDENT
        key, value = key.strip(" '"), value.strip(" '")
        if cur_level < level:  # end of current entity (indent decreases)
            return cur_entity, line
        if cur_level == level:
            if not value:  # got a sub-entity header (i.e. children[x]: or vdev_tree: etc")
                data, next_str = zdb_parse(level + 1, lines)  # recursively analyze this sub-entity
                if '[' in key:  # sub-entity of LIST type (children), repeated with index [0], [1], ..
                    real_key = key[:key.index('[')]
                    if not cur_entity.get(real_key):
                        cur_entity[real_key] = []
                    cur_entity[real_key].append(data)
                else:  # just named sub-entity
                    cur_entity[key] = data
            else:
                if key in NEEDED_KEYS:  # filtering out unneeded info
                    cur_entity[key] = value


def check_vdev(vdev: dict, cache_pool_guid: str, lvl: int = 4):
    """
    Recursively checks given VDEV for correctness:
        type "disk" for disk presence and metadata validity (disk uuid, pool uuid)),
        other types based on their children validity. ZIL is like other VDEVs just with 'is_log" flag set.
        Cache and spare devs are not included in metadata, its absence does not affect to pool import.
    :return: True if given vdev is valid (ok or degraded)
    """
    spacing = ' ' * lvl
    fprint(f'{spacing}VDEV type {vdev.get("type")} ID={vdev.get("guid")} ', end='')
    if not (vdev.get('type') and vdev.get('guid')):
        fprint('No type or guid fields!')
        return False
    if vdev['type'] in ('disk', 'file'):
        if not vdev.get('path'):
            fprint('No path field!')
            return False
        fprint(f'{vdev["path"]} ', end='')
        starttime = time.monotonic()
        while not os.path.exists(vdev['path']):
            time.sleep(0.5)
            fprint('.', end='')
            if time.monotonic() - starttime > DEV_WAIT_TIMEOUT:
                fprint('Error: No device')
                return False
        try:
            cmd_out = subprocess.check_output(['zdb', '-l', vdev['path']], shell=False, timeout=10,
                                              stderr=subprocess.STDOUT).decode()
        except subprocess.CalledProcessError as e:
            fprint(f'ZDB error: {e.returncode} ({e.output.decode().strip()})')
            return False
        disk_dev_guid = disk_pool_guid = ''
        for line in cmd_out.splitlines():
            if line.startswith('    guid:'):
                disk_dev_guid = line[9:].strip()
            elif line.startswith('    pool_guid:'):
                disk_pool_guid = line[14:].strip()
            if disk_dev_guid and disk_pool_guid:
                break
        if disk_dev_guid == vdev['guid'] and disk_pool_guid == cache_pool_guid:
            fprint('OK')
            return True
        else:
            if disk_dev_guid != vdev['guid']:
                fprint(f'{spacing}dev GUID mismatch: ({disk_dev_guid}) on disk != ({vdev["guid"]}) from cache')
            if disk_pool_guid != cache_pool_guid:
                fprint(f'{spacing}pool GUID mismatch: ({disk_pool_guid}) on disk != pool id ({cache_pool_guid}) '
                       f'from cache')
            return False
    else:
        if vdev['type'] == 'hole':  # removed disk
            fprint(f'{spacing} OK')
            return True
        if not vdev.get('children'):
            fprint(f'{spacing}No children!')
            return False
        max_fail = 0  # by default all children must be ok
        if vdev['type'] == 'raidz':
            if not vdev.get('nparity'):
                fprint(f'{spacing}No nparity field for raidz vdev!')
                return False
            max_fail = int(vdev['nparity'])
        elif vdev['type'] == 'mirror':
            max_fail = len(vdev['children']) - 1
        fprint(f'{spacing} <= {max_fail} children can be failed')
        fprint(f'{spacing} Analyzing children:')
        failed = 0
        for child in vdev['children']:
            if not check_vdev(child, cache_pool_guid, lvl + 4):
                failed += 1
                if failed > max_fail:
                    fprint(f'{spacing}FAIL')
                    return False
        fprint(f'{spacing}{"DEGRADED" if failed else "OK"}')
        return True


if __name__ == "__main__":

    if not os.path.isfile(ZPOOL_CACHE):
        print('Nothing to do -- no cache file')
        sys.exit(0)

    if os.stat(ZPOOL_CACHE).st_size == 0:
        print('Cache file is empty!')
        sys.exit(0)

    status, db = subprocess.getstatusoutput('zdb')
    if status != 0:
        fprint(f'ZDB error {status} ({db})')
        sys.exit(1)

    data, _ = zdb_parse(0, db.splitlines().__iter__())
    # print(json.dumps(data, indent=4))

    # checking drives for presence and conformity (checking ZFS device guid from device label)

    with open(ZPOOL_CACHE, 'rb') as cf:
        cache = cf.read()

    for pool_name, pool_data in data.items():
        fprint(f'ZPOOL {pool_name} ID {pool_data["pool_guid"]}')
        for vdev in pool_data['vdev_tree']['children']:
            if not check_vdev(vdev, pool_data['pool_guid']):
                pool_data['failed'] = True
                fprint(f'ZPOOL {pool_name} FAILED')
                break
        if pool_data.get('failed'):
            continue
        fprint(f'ZPOOL {pool_name}: Status OK; ', end='')
        try:
            output = subprocess.check_output(['zpool', 'import', pool_name], shell=False, timeout=30,
                                             stderr=subprocess.STDOUT)
        except subprocess.CalledProcessError as e:
            fprint(f'import FAIL. Error: {e.returncode} ({e.output.decode().strip()})')
        else:
            fprint('import OK')

    with open(ZPOOL_CACHE, 'wb') as cf:
        # if some pools were not imported they are removed from cachefile so we're not able to import
        # it on next reboot automatically even if problems with drives are solved. As workaround, we keep
        # an old cachefile after import. Although, manipulations with zpools during system uptime
        # will remove records about these zpools too, but we're not able to do anything with it
        cf.write(cache)

    sys.exit(0)
