#!/bin/sh

deb_subr="/usr/share/veil/deb.subr"

set -e
. ${deb_subr}
. /etc/veil/global.conf
set +e

if [ -r /lib/systemd/system/openvswitch-switch.service ]; then
  OVS_SWITCH="openvswitch-switch"
else
  OVS_SWITCH="openvswitch"
fi

DEB_LOGFILE="${BASE_LOGS_DIR}/auto_configure_nic.log"

# /var/lib/ecp-veil/utils/other/auto_configure_nic.sh -a init -f -m static -i %s -h %s -t 3
usage()
{
	echo "Usage: $0 -a ACTION [-f] -h hwaddr -i ip -m mode -t timeout -v vagrant_mgmt_iface"
	echo "If there is a network interface with address 00:00:00:00:*, skip configure: sign of the vagrant"
	echo " -a ACTION, can be: init, required. e.g: -a init"
	echo " -d, DNS for nameserver ( comma-separated as delimeter for multiple ): e.g: -d 8.8.8.8 , -d 8.8.8.8,8.8.4.4"
	echo " -f, FORCE re-initialization (for Vagrant mostly), opt, e.g: -f"
	echo " -h, HWADDR, set mac address for veil-default, e.g: -h 000000000001"
	echo " -g, GW, for static mode"
	echo " -i, IP address for static mode, e.g: -i 192.168.0.2"
	echo " -m, mgmt-veil mode: DHCP or STATIC, can be: [static|dhcp], e.g: -m static"
	echo " -t, timeout before reboot when config was changed, in seconds, e.g: -t 3"
	echo " -u, uplink interface, by default unset (= auto), e.g: -u ens14"
	echo " -w, VLANID for veil-mgmt"
	echo " -s, IPMASK for static mode, by default: 255.255.255.0"
	echo " -v, vagrant management interface, by default unset. e.g: -v ens14. Will be sets in in dhcp"
	exit 1
}

flush_cfg()
{
  LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} function - flush_cfg - started${NORMAL}"
	. /usr/share/openvswitch/scripts/ovs-lib

	# protection from unit changes
	systemctl daemon-reload || true
	systemctl stop ${OVS_SWITCH} || true
	ovs_ctl stop || true
	find /etc/openvswitch/ \( -type f -or -type l \) -delete;
	ovs_ctl start --system-id=random
	LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} function - flush_cfg - finished${NORMAL}"
}

generate_interfaces_config()
{
  LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} function - generate_interfaces_config - started${NORMAL}"

  local UPLINK_MTU=$1
	cat > /etc/network/interfaces << EOF
# Generated by SpaceVM node auto_configure_nic.sh
# This file describes the network interfaces available on your system
# and how to activate them. For more information, see interfaces(5).

# The loopback network interface
auto lo
iface lo inet loopback

# The primary network interface
auto ${UPLINK}
iface ${UPLINK} inet manual
    post-up ip link set dev ${UPLINK} mtu ${UPLINK_MTU}

EOF


if [ -n "$VAGRANT_MGMT" ]; then
	cat >> /etc/network/interfaces <<EOF
auto ${VAGRANT_MGMT}
iface ${VAGRANT_MGMT} inet dhcp
EOF
fi

# скрипт нужен в любом случае (с шебангом), не зависимо, dhcp это или
# статика (ниже она заполняет файл с route).
[ ! -d ${NODE_DIR} ] && mkdir -p ${NODE_DIR}
[ ! -d ${NODE_DIR}/bin ] && mkdir -p ${NODE_DIR}/bin
if [ ! -r ${NODE_DIR}/bin/route.sh ]; then
cat > ${NODE_DIR}/bin/route.sh <<EOF
#!/bin/bash
# managed by postinst script
EOF
fi

chmod +x ${NODE_DIR}/bin/route.sh

	case "${MODE}" in
		static)

		if $( echo ${IPADDR} | grep -q '/' ); then
			ip_prefix=1
			IPMASK=
		else
			ip_prefix=0
			[ -z "${IPMASK}" ] && IPMASK="255.255.255.0"
		fi

			cat >> /etc/network/interfaces <<EOF
auto ${MGMT_INTERFACE}
iface ${MGMT_INTERFACE} inet static
    address ${IPADDR}
EOF
			if [ -n "$IPMASK" ]; then
				cat >> /etc/network/interfaces <<EOF
    netmask ${IPMASK}
EOF
			fi
			if [ -n "$DNS" ]; then
				OIFS="$IFS"
				IFS=","
				for i in $DNS; do
					IFS="$OIFS"
					cat >> /etc/resolvconf/resolv.conf.d/base <<EOF
nameserver ${i}
EOF
					IFS=","
				done
				IFS="$OIFS"
			fi
			
			if [ -n "$GW" ]; then
				cat >> /etc/network/interfaces <<EOF
    gateway ${GW}
EOF
			fi
			;;
		*)
			# dhcp by default
			configure_dhclient
			cat >> /etc/network/interfaces <<EOF
auto $MGMT_INTERFACE
iface $MGMT_INTERFACE inet dhcp
EOF
			;;
	esac
	LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} function - generate_interfaces_config - finished${NORMAL}"

}

flush_dhclient_lease_pid()
{
	LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} function - flush_dhclient_lease_pid - started${NORMAL}"

  local infname=$1
  local dhclient_lease_inf_file="/var/lib/dhcp/dhclient.$infname.leases"
  local dhclient_lease_file="/var/lib/dhcp/dhclient.leases"

  ifdown $infname
  if [ -f "/var/run/dhclient.$infname.pid" ]; then
    kill "$(cat "/var/run/dhclient.$infname.pid")"
    rm "/var/run/dhclient.$infname.pid"
  fi

  if [ -f "/var/run/dhclient.pid" ]; then
    kill "$(cat "/var/run/dhclient.pid")"
    rm "/var/run/dhclient.pid"
  fi

  if [ -f $dhclient_lease_inf_file ]; then
    rm $dhclient_lease_inf_file
  fi

  if [ -f $dhclient_lease_file ]; then
    rm $dhclient_lease_file
  fi

	LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} function - flush_dhclient_lease_pid - finished${NORMAL}"
}

configure_dhclient()
{
	LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} function - configure_dhclient - started${NORMAL}"

	local _dhclient='/etc/dhcp/dhclient.conf'
	local _timeout="timeout 20"
	local _req_routers="interface \"$MGMT_INTERFACE\" { also request routers; }"
	local _command="$( grep "$_req_routers" $_dhclient )"

	[ ! -r "$_dhclient" ] && return 0
	[ -z "$_command"  ] && sed -i "1s/^/$_timeout;\n/" $_dhclient && echo $_req_routers >> $_dhclient

	LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} function - configure_dhclient - finished${NORMAL}"
}

add_blackhole_ovs_br()
{
	LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} function - add_blackhole_ovs_br - started${NORMAL}"

	. /usr/share/openvswitch/scripts/ovs-lib
	ovs-vsctl add-br ${BLACKHOLE_OVS}
	ovs-ofctl add-flow ${BLACKHOLE_OVS} 'table=0,action=drop'

	LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} function - add_blackhole_ovs_br - finished${NORMAL}"
}

add_default_ovs_br()
{
	LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} function - add_default_ovs_br - started${NORMAL}"

	. /usr/share/openvswitch/scripts/ovs-lib
	ovs-vsctl add-br ${DEFAULT_OVS}
  # Генерация мака, чтоб lshw при установке на vmware видела этот bridge, иначе он берет мак с физики
  DEFAULT_BR_MAC=$( generate_mac ${VEIL_OUI} )
  ovs-vsctl set bridge ${DEFAULT_OVS} other-config:hwaddr=\"${DEFAULT_BR_MAC}\"
	ovs-vsctl add-port ${DEFAULT_OVS} ${UPLINK}
	ovs-vsctl set port ${UPLINK} vlan_mode=trunk

	LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} function - add_default_ovs_br - finished${NORMAL}"
	LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} default uplink interface ${UPLINK} connected to ${DEFAULT_OVS}${NORMAL}"
}

add_mgmt_interface()
{
  LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} function - add_mgmt_interface - started${NORMAL}"
	. /usr/share/openvswitch/scripts/ovs-lib

	local _MAC

  if [ -n "${HWADDR}" ]; then
    _MAC="${HWADDR}"
  else
    _MAC=$( generate_mac ${VEIL_OUI} )
  fi

	ovs-vsctl add-port ${DEFAULT_OVS} ${MGMT_INTERFACE} -- set interface ${MGMT_INTERFACE} type=internal mac=\""${_MAC}"\"
	ovs-vsctl set port ${MGMT_INTERFACE} vlan_mode=access
	ovs-vsctl set int ${MGMT_INTERFACE} mtu_request=${UPLINK_MTU}
	generate_interfaces_config $UPLINK_MTU

	if [ -n "${VLANID}" ]; then
    ovs-vsctl set port ${MGMT_INTERFACE} tag=${VLANID}
    LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} ${MGMT_INTERFACE} VLANID is set to ${VLANID}${NORMAL}"
  fi

	LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} function - add_mgmt_interface - finished${NORMAL}"
	LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} management interface MAC-address is set to ${_MAC}${NORMAL}"
}

# force modify
FORCE=0

# vagrant attribute
IS_VAGRANT=0

#
MODIFIED=0

# timeout before reboot when cfg is changed
REBOOT_TIMEOUT=10

# veil-mgmt IP address mode by default: dhcp
MODE="dhcp"

# hwaddr random, if unset
HWADDR=

# ip addr by default: unset
IPADDR=

# uplink by default: unset
UPLINK=

# ipmask by default: unset
IPMASK=

# gw by default: unset
GW=

# DNS by default: unset
DNS=

# vagrant management interface by default: unset
VAGRANT_MGMT=

# VLANID for veil-mgmt: unset
VLANID=

# -a ACTION, can be: init, required. e.g: -a init
# -d, DNS for nameserver ( comma-separated as delimer for multiple ): e.g: -d 8.8.8.8 , -d 8.8.8.8,8.8.4.4
# -f, FORCE re-initialization (for Vagrant mostly), opt, e.g: -f
# -h, HWADDR, set MAC address for mgmt-vail, e.g: -h 000000000001
# -g, GW, set gateway for static mode
# -i, IP address for static mode, e.g: -i 192.168.0.2
# -m, mgmt-veil mode: DHCP or STATIC, can be: [static|dhcp], e.g: -m static
# -t, timeout before reboot when config was changes, in seconds, e.g: -t 3
# -u, set UPLINK to specified interface
# -s, set mask for static mode, by default: 255.255.255.0
# -v, vagrant management interface, by default unset. e.g: -v ens14. Will be sets in in dhcp"

while getopts "a:d:fh:g:i:m:t:u:s:v:w:" opt; do
	case "$opt" in
		a) ACTION="${OPTARG}" ;;
		d) DNS="${OPTARG}" ;;
		f) FORCE=1 ;;
		g) GW="${OPTARG}" ;;
		h) HWADDR="${OPTARG}" ;;
		i) IPADDR="${OPTARG}" ;;
		m) MODE="${OPTARG}" ;;
		t) REBOOT_TIMEOUT="${OPTARG}" ;;
		u) UPLINK="${OPTARG}" ;;
		s) IPMASK="${OPTARG}" ;;
		v) VAGRANT_MGMT="${OPTARG}" ;;
    w) VLANID="${OPTARG}" ;;
	esac
done
shift $(($OPTIND - 1))

[ "${ACTION}" != "init" ] && usage

[ -z "${IPMASK}" ] && IPMASK="255.255.255.0"

[ "${BRIDGE_AUTO_CONFIGURE}" != "1" ] && exit 0
MY_APPLICATION="auto_configure_nic"

# make backup
DT=$( date "+%Y-%m-%d-%H-%M-%S" )
[ -r /etc/network/interfaces ] && cp /etc/network/interfaces /etc/network/interfaces-${DT}

if [ "$MODE" = "static" ]; then

	[ -z "$IPADDR" ] && IPADDR=$( grep address /etc/network/interfaces | awk '{print $2}' )
	[ -z "$GW" ] && GW=$( grep gateway /etc/network/interfaces | awk '{print $2}' )
	[ -z "$DNS" ] && DNS=$( grep nameserver /etc/network/interfaces | awk '{$1=""; print $0}' | awk '{for(i=1;i<NF;i++)if(i!=NF){$i=$i","}}1' | sed 's/ //g' )
	LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} static ip address ${IPADDR} with gateway ${GW} and dns ${DNS} ${NORMAL}"
elif [ "$MODE" = "dhcp" ]; then
  LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} dynamic ip address${NORMAL}"
  flush_dhclient_lease_pid $UPLINK
fi

[ "$MODE" = "static" -a -z "$IPADDR" ] && err 1 "${RED}${MY_APPLICATION} error: ${LCYAN}when mode static, ip must be set, please use -i IP${NORMAL}"

# last octet is dynamic from Vagrantfile
if $( ip a | grep 'link/ether' | grep -q 00:00:00:00: ); then
	IS_VAGRANT=1
fi

if [ "$FORCE" -eq 1 ]; then
	LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} flushing old openvswitch config by force flag${NORMAL}"
	flush_cfg
	MODIFIED=1
fi

# test
systemctl start ${OVS_SWITCH}.service || true

if $( ovs-vsctl show | grep -q "${BLACKHOLE_OVS}" ); then
	LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE}${BLACKHOLE_OVS} already configured${NORMAL}"
else
	LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} configuring ${BLACKHOLE_OVS}${NORMAL}"
	add_blackhole_ovs_br
	MODIFIED=1
fi

if [ -n "$UPLINK" ]; then
	UPLINK="$UPLINK"
	LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} uplink interface ${UPLINK} from -u arg${NORMAL}"
else
	UPLINK=$( get_default_uplink )
	LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} uplink interface ${UPLINK} got by default${NORMAL}"
fi

UPLINK_MTU="$(cat /sys/class/net/${UPLINK}/mtu)"
LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} uplink interface ${UPLINK} has mtu ${UPLINK_MTU}${NORMAL}"

if $( ovs-vsctl show | grep -q "${DEFAULT_OVS}" ); then
	LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE}${DEFAULT_OVS} interface already configured, skip${NORMAL}"
else
	LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} configuring ${DEFAULT_OVS}${NORMAL}"
	add_default_ovs_br
	MODIFIED=1
fi

if $( ovs-vsctl show | grep -q "${MGMT_INTERFACE}" ); then
	LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE}${MGMT_INTERFACE} interface already configured, skip${NORMAL}"
	exit 0
else
	LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} configuring ${MGMT_INTERFACE}${NORMAL}"
	add_mgmt_interface
	MODIFIED=1
fi

LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} modified status: ${MODIFIED}${NORMAL}"

if [ "$MODIFIED" -eq 1 ]; then
  LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} System was modified and reconfigured${NORMAL}"
	/usr/local/bin/update-veil-issue.sh
fi

if [ "$IS_VAGRANT" -eq 0 ]; then
  LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} Start last check and transform${NORMAL}"
  systemctl restart ${OVS_SWITCH}.service
  LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} restart ${OVS_SWITCH}.service errcode: $?${NORMAL}"

  echo "from veilrpc.states.networking import transform_network_config; transform_network_config()" | ${CLI_ENV_DIR}/bin/python
  LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} transform_network_config with errcode $?${NORMAL}"

  ifdown "${UPLINK}" && ifup "${UPLINK}" --ignore-errors
  LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} ifdown ${UPLINK} && ifup ${UPLINK} --ignore-errors errcode: $?${NORMAL}"

  ifdown "${MGMT_INTERFACE}" && ifup "${MGMT_INTERFACE}" --ignore-errors
  LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} ifdown ${MGMT_INTERFACE} && ifup ${MGMT_INTERFACE} --ignore-errors errcode: $?${NORMAL}"

  LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} Exit without reboot${NORMAL}"
  [ -d "${BASE_DIR}/tests" ] && touch "${BASE_DIR}/tests/net_configured"
else
  LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} vagrant mode. only transform_network_config on controller${NORMAL}"
  controller=0
  node=0

  if [ ! -f "$VEIL_MEDIA_CONF" ]; then
    LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} No such ${VEIL_MEDIA_CONF} ${NORMAL}"
  else
    . ${VEIL_MEDIA_CONF}
    LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} controller ${controller} and node ${node} ${NORMAL}"
  fi

  if [ ${controller} -eq 1 -a ${node} -eq 0 ]; then
    echo "from veilrpc.states.networking import transform_network_config; transform_network_config()" | ${CLI_ENV_DIR}/bin/python
    LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} transform_network_config with errcode $?${NORMAL}"
  fi
  [ -d "${BASE_DIR}/tests" ] && touch "${BASE_DIR}/tests/net_configured"

  while [ "$REBOOT_TIMEOUT" -gt 1 ]; do
    echo -n "the system will reboot after ${REBOOT_TIMEOUT} seconds, press Ctrl+C to cancel"
    sleep 1
    printf "${CLRLINE}"
    printf "${CURSORRST}"
    REBOOT_TIMEOUT=$(( REBOOT_TIMEOUT - 1 ))
  done

  LOG "* ${LCYAN}${MY_APPLICATION}: ${WHITE} Reboot in vagrant mode${NORMAL}"
  reboot
fi
